#!/usr/bin/perl
#
# dr_align_efsg: do alignment using efsgd
#
# $Header: /u/drspeech/src/dr_scripts/RCS/dr_align_efsg.pm,v 1.13 2000/02/16 21:24:07 fosler Exp fosler $

package dr_align_efsg;
use drspeech;
use drspeech_args;
use gen_efsg;
use dr_align;

# align(): perform alignment
sub align {
    $args=[["verbose","Verbosity level",$drspeech_args::pattern{"integer"},1],
	   ["activation_file","The source of the input probabilities",
	    $drspeech_args::pattern{"filename"},"-"],
	   ["activation_format","The format of the input probability stream",
	    "(rapbin)|(lna)","rapbin"],
	   ["utt_words_file","The file containing the target word transcriptions",
	    $drspeech_args::pattern{"filename"}],
	   ["utt_words_file_type","The file type of the target word transcriptions",
	    "(stm)|(ref)|(tagref)","ref"],
	   ["utt_start","The first sentence to align",
	    $drspeech_args::pattern{"integer"},0],
	   ["utt_start_ref","The first sentence to take from the ref file, relative to utt_start",
	    $drspeech_args::pattern{"integer"},0],
	   ["utt_count","The number of sentences to align (<1 means all)",
	    $drspeech_args::pattern{"integer"},0],
	   ["phi","phone information file",
	    $drspeech_args::pattern{"opt_filename"},""],
	   ["phone_models","noway phone models file",
	    $drspeech_args::pattern{"opt_filename"},""],
	   ["priors","priors file",
	    $drspeech_args::pattern{"opt_filename"},""],
	   ["ctm_file","The alignment data is written to this file",
	    $drspeech_args::pattern{"filename"}],
	   ["ctm_type","Type of ctm output",
	    "(phone)|(word)|(phone[+]word)","phone+word"],
	   ["ctm_overwrite","Allow overwrite of ctm file if it exists",
	    $drspeech_args::pattern{"boolean"},"false"],
	   ["valid_utts","File to write containing all valid utterance numbers",
	    $drspeech_args::pattern{"filename"},"valid_utts.$$"],
	   ["ftr_pfile","Pfile containing the features used to generate the activations",
	    $drspeech_args::pattern{"opt_filename"},""],
	   ["steptime","The step between each analysis frame in ms",
	    $drspeech_args::pattern{"float"},10.0],
	   ["zeropad","The amount of zeropadding in the ftr pfile relative to the label pfile in ms",
	    $drspeech_args::pattern{"float"},40.0],
	   ["dictionary","dictionary for looking up in-vocabulary forms",
	    $drspeech_args::pattern{"opt_filename"},""],
	   ["treelist","name of file containing list of trees",
	    $drspeech_args::pattern{"opt_filename"},""],
	   ["silprob","probability of interword silence",
	    $drspeech_args::pattern{"float"},0.001],
	   ["silwrd","silence word label",
	    $drspeech_args::pattern{"any"},"<SIL>"],
	   ["silphn","silence phone label",
	    $drspeech_args::pattern{"any"},"h#"],
	   ["fragment_samplect",
	    "Number of random words to determine fragment pron from ",
	    $drspeech_args::pattern{"integer"},10],
	   ["random_seed","seed for initializing random number generator",
	    $drspeech_args::pattern{"float"},0],
	   ["prunelevel","Prune phones with less than this probability",
	    $drspeech_args::pattern{"float"},0.01],
	   ["alphabet","Allowable characters in the alphabet",
	    $drspeech_args::pattern{"any"},"A-Za-z\\-\\'\\.\\#"],
	   ["efsgd_prog","path to efsgd executable",
	    $drspeech_args::pattern{"filename"},"efsgd"],
	   ["efsgd_opt_args","optional arguments to efsgd",
	    $drspeech_args::pattern{"any"},""]];

    $found=&drspeech_args::parse($args,@_);

    if ($dictionary eq "" && $treelist eq "") {
	die("dr_align_efsg::align: must define one of dictionary or treelist");
    }
    if (($phi eq "" && ($phone_models eq "" || $priors eq "")) ||
	($phi ne "" && ($phone_models ne "" || $priors ne "")) ||
	($phone_models ne "" && $priors eq "") ||
	($phone_models eq "" && $priors ne "")) {
	die("dr_align_efsg::align: must define one of (a) phi file or (b) phone_models and priors file");
    }

    $allow_badsegs=1;
    if ($ftr_pfile eq "") {
	if ($utt_start != 0) {
	    die("dr_align_efsg::align: if no ftr_pfile is specified, utt_start must be 0");
	}
	warn("dr_align_efsg::align: ftr_pfile is not specified, will be unable to recover from any bad alignments");
	$allow_badsegs=0;
    }

    srand($random_seed);

    @passargs=("verbose=$drspeech::verbose",
	       "treelist=$treelist",
	       "silprob=$silprob",
	       "silwrd=$silwrd",
	       "silphn=$silphn",
	       "fragment_samplect=$fragment_samplect",
	       "random_seed=$random_seed",
	       "dict=$dictionary",
	       "prunelevel=$prunelevel",
	       "alphabet=$alphabet");

    $gen=new gen_efsg(@passargs);

    #set up phi and phndict files
    if ($phi eq "") {
	$phi=$drspeech::tmpdir."/drspeech_dr_align_efsg_phi.$$";
#	&dr_align_utils::phmodels_and_priors_2_phi($phmodels,$priors,$phi);
    }
    $phndict=$drspeech::tmpdir."/drspeech_dr_align_efsg_phndict.$$";
    $nphones=&phi2phndict($phi,$phndict);

    if ($utt_count < 1) {
	$utt_end="";
    } else {
	$utt_end=($utt_start+$utt_count)-1;
    }

    # make a fifo for lna input to efsgd, as we intend to use efsgd's
    # stdin for grammar input
    $feacat_pipe=$drspeech::tmpdir."/drspeech_dr_align_efsg_feacat_pipe.$$";
    $ENV{PATH} .= ":/etc:/usr/etc";
    if  (system('mknod',  $feacat_pipe, 'p') &&
	 system('mkfifo', $feacat_pipe)) {
	die "mk{nod,fifo} $feacat_pipe failed";
    }

    # set up log files
    $feacat_log=$drspeech::tmpdir."/drspeech_dr_align_efsg_feacat_log.$$";
    $efsg_log=$drspeech::tmpdir."/drspeech_dr_align_efsg_efsgd_log.$$";

    # Do we need feacat?
    if ($utt_start eq "0" && $utt_end eq "" && $activation_format eq "lna" && $activation_file ne "-") {
	# No, just read the input lna
	$lna_source = $activation_file;
	if ($drspeech::verbose >= 1) {
	    &drspeech::inform("reading activation file directly\n");
	}
	if ($activation_file =~ m/.*\.gz/) {
	    if ($drspeech::verbose >= 1) {
		&drspeech::inform(".. via gunzip -c\n");
	    }
	    $lna_source = $feacat_pipe;

	    $command="gunzip -c \
              $activation_file \
              > $feacat_pipe \
              2> $feacat_log";
	    $command =~ s/ [ \n]*/ /go;
	
	    # fork off the feacat process (actually gzip), but save the pid
	    unless ($feacat_pid = fork) {
		exec "$command";
		die "no exec";
		exit 0;
	    }
	}
	$feacat_pid = 0;
    } else {
	$lna_source = $feacat_pipe;

	$command="feacat \
              -ipformat $activation_format \
              -opformat lna \
              -width $nphones \
              -sr $utt_start:$utt_end \
              $activation_file \
              > $feacat_pipe \
              2> $feacat_log";
	$command =~ s/ [ \n]*/ /go;
	
	if ($drspeech::verbose>=2) {
	    &drspeech::inform($command,"\n");
	}

	# fork off the feacat process, but save the pid
	unless ($feacat_pid = fork) {
	    exec "$command";
	    die "no exec";
	    exit 0;
	}
    }

    $tmp_ctm=$drspeech::tmpdir."/drspeech_dr_align_efsg_ctm.$$";

    if ($allow_badsegs) {
	$output_ctm = $tmp_ctm;
    } else {
	$output_ctm = $ctm_file;
    }

    if ($drspeech::verbose >= 2) {
	$efsgd_opt_args = $efsgd_opt_args . " -debug";
	$grammar_debug_log = $drspeech::tmpdir."/drspeech_dr_align_efsg_grammar_log.$$";
	$command = "tee $grammar_debug_log | ";
    } else {
	$command = "";
    }

    $command .= "$efsgd_prog \
                 -extGrammar \
                 -phi $phi \
                 -dct $phndict \
                 -ctm $output_ctm \
                 -frameShift $steptime \
                 $efsgd_opt_args \
                 $lna_source \
                 - \
                 /dev/null \
                 2> $efsg_log";

    $command =~ s/ [ \n]*/ /go;

    if ($drspeech::verbose>=2) {
	&drspeech::inform($command,"\n");
    }

    open(REF,"$utt_words_file") || die("dr_align_efsg::align: Can't open $utt_words_file");
    open(EFSGD,"| $command") || die("Can't open $command");
    select(EFSGD); $|=1; select(STDOUT);

    if ($drspeech::verbose>=2 && $utt_start+$utt_start_ref > 0) {
	&drspeech::inform(sprintf("skipping ref sents 0..%d\n",$utt_start+$utt_start_ref-1));
    }
    for($utt=0;$utt<$utt_start+$utt_start_ref;$utt++) {
	if (! &get_next_utt(\*REF,$utt_words_file_type)) {
	    die("dr_align_efsg::align: fewer utterances in $utt_words_file than specified (tried to read utt $utt)");
	}
    }

    $utt_end=undef if $utt_end eq "";
    for($utt=$utt_start+$utt_start_ref;!defined($utt_end) || $utt<=$utt_end+$utt_start_ref; $utt++) {
	if ($drspeech::verbose>=2) {
	    print STDERR "reading sent $utt...";
	}
	if (! (@ret=&get_next_utt(\*REF,$utt_words_file_type))) {
	    if (defined($utt_end)) {
		die("dr_align_efsg::align: fewer utterances in $utt_words_file than specified (tried to read utt $utt)");
	    } else {
		# Assume it's just the end of the utterances
		# back off utt count to get true end
		if ($drspeech::verbose>=2) {
		    print STDERR "ref eof seen\n";
		}
		$utt--;
		# end the loop
		last;
	    }
	}
	($_,$id)=@ret;
	chop;
	if ($drspeech::verbose>=3) {
	    print STDERR "\n$_\n";
	}
	@words=split;
#       print STDERR "WORDS: ",join(" ",@words),"\n";
	if ($drspeech::verbose>=2) {
	    printf STDERR ("creating grammar %s...",defined($id)?" (".$id.")":"");
	}
	@efsg_lines=$gen->get_efsg(@words);
	print EFSGD @efsg_lines;
	if ($drspeech::verbose>=2) {
	    print STDERR "done.\n";
	}
#	print STDERR @efsg_lines;
    }
    $utt_end=$utt-$utt_start_ref;
    close(REF);
    close(EFSGD);
    $efsg_status=$?;

    if ($feacat_pid != 0) {
	waitpid($feacat_pid,0);
	$feacat_status=$?;
    } else {
	$feacat_status = 0;
    }

    @badutts=();
    @baduttsindx=();
    $bigerror=0;

    if ($efsg_status) {
	if (open(EFSG_LOG,$efsg_log)) {
	    while(<EFSG_LOG>) {
		chop;
		if ($verbose>=2) {
		    &drspeech::inform("efsg_log: ",$_,"\n");
		}
		if (/Utterance ([0-9]+): no path to traceback$/) {
		    s/.*Utterance //;
		    s/: no path to traceback$//;
		    $uttnum=$_;
		    
		    push(@badutts,$uttnum);
		    @baduttsindx[$uttnum]=1;
		} else {
		    $bigerror=1;
		}
	    }
	    close(EFSG_LOG);
	} else {
	    $bigerror=1;
	}
	if ($bigerror) {
	    warn("dr_align_efsg::align: efsgd returned with errors other than non-alignments (status $efsg_status)");
	} else {
	    warn("dr_align_efsg::align: efsgd was not able to align all sentences");
	}
    } 
    if ($feacat_status) {
	if ($verbose>=2 && open(FEACAT_LOG,$feacat_log)) {
	    while(<FEACAT_LOG>) {
		chop;
		&drspeech::inform("feacat_log: ",$_,"\n");
	    }
	}
	warn("dr_align_efsg::align: feacat returned with status $feacat_status");
    } 

    # temporary measure
    if ($ctm_type ne "phone+word") {
	warn("dr_align_efsg::align: only phone+word CTM files supported at the moment, sorry!");
	$ctm_type="phone+word";
    }

    # declare all utts before utt_start bad
    # remember that CTM utts are 1-indexed, but utt_start is 0-indexed
    for($i=1;$i<=$utt_start;$i++) {
	$baduttsindx[$i]=1;
	push(@badutts,$i);
    }

    if ($ctm_type eq "phone+word") {
	if ($allow_badsegs) {
	    if ($#badutts<0) { # no bad utterances!
		system("mv $tmp_ctm $ctm_file");
		open(VALIDUTTS,">$valid_utts") || warn("dr_align_efsg::align: can't open $valid_utts for writing: $!");
		printf VALIDUTTS "all\n";
		close(VALIDUTTS);
	    } else {
		die("dr_align_efsg::align: some sentences didn't align, and no ftr_pfile to recover from") if ($ftr_pfile eq "");
		&ctm_fill_empty_utts($tmp_ctm,$ctm_file,$ftr_pfile,$steptime,$zeropad,\@badutts,1,$utt_start);
		open(VALIDUTTS,">$valid_utts") || warn("dr_align_efsg::align: can't open $valid_utts for writing: $!");
		$state=0;
		@ranges=();
		# remember that CTM utts are 1-indexed, but utt_start is 0-indexed
		for($i=$utt_start+1;$i<=$utt_end+1;$i++) {
		    if (!defined($baduttsindx[$i]) && $state==0) {
			$start=$i-1;
			$state=1;
		    } elsif (defined($baduttsindx[$i]) && $state==1) {
			$end=$i-2;
			push(@ranges,$start."-".$end);
			$state=0;
		    }
		}
		if ($state==1) {
		    $ue=$utt_end-1;
		    push(@ranges,$start."-".$ue);
		}
		print VALIDUTTS join(",",@ranges),"\n";
		close(VALIDUTTS);
	    }
	} else {
	    # !$allow_badsegs
	    if ($#badutts>=0) {
		die("dr_align_efsg::align: $#badutts+1 segs did not align but cannot recover")
	    }
	    # else you squeaked through
	}
    } else {
	#
    }

    
}

	


1;


