#!/usr/local/bin/perl

package string_aligner;
use drspeech;

# new(): generic new operator for blessing
sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = {};
    bless $self, $class;
    if (defined($_[0])) {
	$self->set_metric(@_);
    } else {
	$self->set_metric("default");
    }
    return $self;
}

sub set_metric {
    my $self=shift(@_);

    if (!ref($_[0])) {
	for ($_[0]) {
	    (/^default$/ || /^basic$/) && do { 
		$self->{"costfn"}=\&string_aligner::basic_cost; 
		if ($#_ == 3) {
		    $self->{"cost_sub"}=$_[1];
		    $self->{"cost_del"}=$_[2];
		    $self->{"cost_ins"}=$_[3];
		} else {
		    $self->{"cost_sub"}=1.0;
		    $self->{"cost_del"}=1.2;
		    $self->{"cost_ins"}=1.2;
		}
		last;
	    };
	    /^phonedist$/ && do {
		$self->{"costfn"}=\&string_aligner::phonedist_cost; last;
	    };
	    die("string_aligner::setmetric: unknown cost type $_[0]\n");
	}
    } else {
	for (ref($_[0])) {
	    
	    /^CODE$/ && do {
		$self->{"costfn"}=$_[0];
		last;
	    };
	    /^HASH$/ && do {
		$self->{"costmatrix"}=$_[0];
		$self->{"costfn"}=\&string_aligner::lookup_cost;
		last;
	    };
	    die("string_aligner::setmetric: unknown type for argument 0: ".ref($_[0])."\n");
	}
    }
}

# do dynamic programming alignment using some distance metric	      
sub align {
    my $self=shift;
    my @pron1;
    my @pron2;
    my @backtrace;
    # costfn is a pointer to a function that takes two arguments
    # and returns a distance between them
    my $costfn=$self->{"costfn"};
    my ($p1,$p2,$val1,$val2);
    
    #put the arguments in easy-to-use format
    $p1=shift;
    @pron1=@{$p1};
    $p2=shift;
    @pron2=@{$p2};

    # move down the columns
    for($p1=0;$p1<=$#pron1+1;$p1++) {
	$curscore[$p1]=[0];
	$curdir[$p1]=[""];
	$val1=$pron1[$p1-1] if $p1>0;
	#move down the rows
	for($p2=0;$p2<=$#pron2+1;$p2++) {
	    $val2=$pron2[$p2-1] if $p2>0;
	    if ($p1==0 && $p2==0) {
		print STDERR "\t" if $drspeech::verbose>=4;
		next;
	    }
	    # scores will contain scores for SUB/CORR vs. DEL vs. INS
	    # directions will contain tags for SUB/CORR/DEL/INS
	    @scores=();
	    @directions=();
	    if ($p1>0) {
		push(@scores,$curscore[$p1-1]->[$p2]+
		     &{$costfn}($self,$val1,"DEL"));
		push(@directions,"DEL");
		if ($p2>0) {
		    if ($pron1[$p1-1] eq $pron2[$p2-1]) {
			# should pass corr score?
			push(@scores,$curscore[$p1-1]->[$p2-1]);
			push(@directions,"COR");
		    } else {
			push(@scores,$curscore[$p1-1]->[$p2-1]+
			     &{$costfn}($self,$val1,$val2));
			push(@directions,"SUB");
		    }
		}
	    }
	    if ($p2>0) {
		push(@scores,$curscore[$p1]->[$p2-1]+
		     &{$costfn}($self,"INS",$val2));
		push(@directions,"INS");
	    }
	    # pick the lowest score
	    @ind=sort {$scores[$a]<=>$scores[$b];} (0..$#scores);
	    $curscore[$p1]->[$p2]=$scores[$ind[0]];
	    $curdir[$p1]->[$p2]=$directions[$ind[0]];
	    print STDERR $directions[$ind[0]]."/".$scores[$ind[0]]."\t" if 
		$drspeech::verbose>=4;
	}
	print STDERR "\n" if $drspeech::verbose>=4;
    }
    # do the backtrace 
    @backtrace=();
    $p1=$#pron1+1;
    $p2=$#pron2+1;
    while($p1>0 || $p2>0) {
	unshift(@backtrace,$curdir[$p1]->[$p2]);
	for($curdir[$p1]->[$p2]) {
	    (/SUB/ || /COR/) && do {$p1--; $p2--; last;};
	    /DEL/ && do {$p1--; last;};
	    /INS/ && do {$p2--; last;};
	    die("Shouldn't get here");
	}
    }
    return @backtrace;
}

sub get_string_score {
    my $self=shift;
    my @pron1;
    my @pron2;
    my @backtrace;
    # costfn is a pointer to a function that takes two arguments
    # and returns a distance between them
    my $costfn=$self->{"costfn"};
    my ($p1,$p2,$val1,$val2);
    
    #put the arguments in easy-to-use format
    $p1=shift;
    @pron1=@{$p1};
    $p2=shift;
    @pron2=@{$p2};

    # move down the columns
    for($p1=0;$p1<=$#pron1+1;$p1++) {
	$curscore[$p1]=[0];
	$curdir[$p1]=[""];
	$val1=$pron1[$p1-1] if $p1>0;
	#move down the rows
	for($p2=0;$p2<=$#pron2+1;$p2++) {
	    $val2=$pron2[$p2-1] if $p2>0;
	    if ($p1==0 && $p2==0) {
		print STDERR "\t" if $drspeech::verbose>=4;
		next;
	    }
	    # scores will contain scores for SUB/CORR vs. DEL vs. INS
	    # directions will contain tags for SUB/CORR/DEL/INS
	    @scores=();
	    @directions=();
	    if ($p1>0) {
		push(@scores,$curscore[$p1-1]->[$p2]+
		     &{$costfn}($self,$val1,"DEL"));
		push(@directions,"DEL");
		if ($p2>0) {
		    if ($pron1[$p1-1] eq $pron2[$p2-1]) {
			# should pass corr score?
			push(@scores,$curscore[$p1-1]->[$p2-1]);
			push(@directions,"COR");
		    } else {
			push(@scores,$curscore[$p1-1]->[$p2-1]+
			     &{$costfn}($self,$val1,$val2));
			push(@directions,"SUB");
		    }
		}
	    }
	    if ($p2>0) {
		push(@scores,$curscore[$p1]->[$p2-1]+
		     &{$costfn}($self,"INS",$val2));
		push(@directions,"INS");
	    }
	    # pick the lowest score
	    @ind=sort {$scores[$a]<=>$scores[$b];} (0..$#scores);
	    $curscore[$p1]->[$p2]=$scores[$ind[0]];
	    $curdir[$p1]->[$p2]=$directions[$ind[0]];
	    print STDERR $directions[$ind[0]]."/".$scores[$ind[0]]."\t" if 
		$drspeech::verbose>=4;
	}
	print STDERR "\n" if $drspeech::verbose>=4;
    }
    return $curscore[$#pron1+1]->[$#pron2+1];
}
    
sub get_distance {
    my $self=shift;
    my $ph1=shift;
    my $ph2=shift;
    # costfn is a pointer to a function that takes two arguments
    # and returns a distance between them
    my $costfn=$self->{"costfn"};
   
    return &{$costfn}($self,$p1,$p2);
}

sub basic_cost {
    my $self=shift;
    my $p1=shift;
    my $p2=shift;

    if ($p1 eq $p2) {
	return 0;
    } elsif ($p1 eq "INS") {
	return $self->{"cost_ins"};
    } elsif ($p2 eq "DEL") {
	return $self->{"cost_del"};
    } else {
	return $self->{"cost_sub"};
    }
}

sub phonedist_cost {
    my $self=shift;
    my $p1=shift;
    my $p2=shift;

    return $phone_dist{$p1}->{$p2};
}

sub lookup_cost {
    my $self=shift;
    my $p1=shift;
    my $p2=shift;
    my $cost=$self->{"costmatrix"};

    if (!defined($cost) ||
	!defined($cost->{$p1}) ||
	!defined($cost->{$p1}->{$p2})) {
	die("Can't find cost lookup for $p1 vs $p2");
    }
    return $cost->{$p1}->{$p2};
}

# why did this work this way?  It's no different than basic_cost then.
# JEFL: 20 April 2000
sub lookup_cost_old {
    my $self=shift;
    my $p1=shift;
    my $p2=shift;

    if ($p1 eq $p2) {
	return 0;
    } elsif ($p1 eq "DEL") {
	return $self->{"cost_del"};
    } elsif ($p2 eq "INS") {
	return $self->{"cost_ins"};
    } else {
	return $self->{"cost_sub"};
    }
}

# The following is a distance metric based on the number of Withgott-Chen 
# features different between two phones.  Deletions and Insertions have been
# set to 5 features, which works well empirically.
%phone_dist=(
  "aa" => { "aa"=>0, "ae"=>2, "ah"=>3, "ao"=>3, "aw"=>4, "ax"=>2, "ax-h"=>3, "axr"=>4, "ay"=>4, "b"=>7, "bcl"=>8, "ch"=>12, "d"=>8, "dcl"=>9, "dh"=>9, "dx"=>9, "eh"=>3, "el"=>8, "em"=>6, "en"=>7, "eng"=>5, "epi"=>7, "er"=>6, "ey"=>5, "f"=>11, "g"=>6, "gcl"=>7, "h#"=>7, "hh"=>7, "hv"=>6, "ih"=>4, "ix"=>3, "iy"=>5, "jh"=>11, "k"=>8, "kcl"=>9, "l"=>9, "m"=>7, "n"=>8, "ng"=>6, "nx"=>9, "ow"=>3, "oy"=>3, "p"=>9, "pau"=>7, "pcl"=>10, "q"=>7, "r"=>8, "s"=>13, "sh"=>13, "t"=>10, "tcl"=>11, "th"=>11, "uh"=>3, "uw"=>4, "ux"=>5, "v"=>9, "w"=>8, "y"=>8, "z"=>11, "zh"=>11, "DEL"=>5 },
  "ae" => { "aa"=>2, "ae"=>0, "ah"=>3, "ao"=>5, "aw"=>2, "ax"=>2, "ax-h"=>3, "axr"=>4, "ay"=>2, "b"=>7, "bcl"=>8, "ch"=>12, "d"=>8, "dcl"=>9, "dh"=>9, "dx"=>9, "eh"=>1, "el"=>8, "em"=>6, "en"=>7, "eng"=>7, "epi"=>7, "er"=>6, "ey"=>3, "f"=>11, "g"=>8, "gcl"=>9, "h#"=>7, "hh"=>7, "hv"=>6, "ih"=>2, "ix"=>1, "iy"=>3, "jh"=>11, "k"=>10, "kcl"=>11, "l"=>9, "m"=>7, "n"=>8, "ng"=>8, "nx"=>9, "ow"=>5, "oy"=>5, "p"=>9, "pau"=>7, "pcl"=>10, "q"=>9, "r"=>8, "s"=>13, "sh"=>13, "t"=>10, "tcl"=>11, "th"=>11, "uh"=>5, "uw"=>6, "ux"=>7, "v"=>9, "w"=>10, "y"=>8, "z"=>11, "zh"=>11, "DEL"=>5 },
  "ah" => { "aa"=>3, "ae"=>3, "ah"=>0, "ao"=>2, "aw"=>5, "ax"=>1, "ax-h"=>2, "axr"=>3, "ay"=>5, "b"=>6, "bcl"=>7, "ch"=>9, "d"=>7, "dcl"=>8, "dh"=>8, "dx"=>8, "eh"=>2, "el"=>5, "em"=>5, "en"=>6, "eng"=>6, "epi"=>6, "er"=>3, "ey"=>2, "f"=>8, "g"=>7, "gcl"=>8, "h#"=>6, "hh"=>8, "hv"=>7, "ih"=>3, "ix"=>2, "iy"=>2, "jh"=>8, "k"=>7, "kcl"=>8, "l"=>6, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>2, "oy"=>4, "p"=>6, "pau"=>6, "pcl"=>7, "q"=>8, "r"=>7, "s"=>10, "sh"=>10, "t"=>7, "tcl"=>8, "th"=>8, "uh"=>4, "uw"=>3, "ux"=>4, "v"=>8, "w"=>9, "y"=>7, "z"=>10, "zh"=>10, "DEL"=>5 },
  "ao" => { "aa"=>3, "ae"=>5, "ah"=>2, "ao"=>0, "aw"=>7, "ax"=>3, "ax-h"=>4, "axr"=>3, "ay"=>7, "b"=>8, "bcl"=>9, "ch"=>11, "d"=>9, "dcl"=>10, "dh"=>10, "dx"=>10, "eh"=>4, "el"=>7, "em"=>7, "en"=>8, "eng"=>6, "epi"=>8, "er"=>3, "ey"=>4, "f"=>10, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>10, "hv"=>9, "ih"=>5, "ix"=>4, "iy"=>4, "jh"=>10, "k"=>7, "kcl"=>8, "l"=>8, "m"=>8, "n"=>9, "ng"=>7, "nx"=>10, "ow"=>0, "oy"=>2, "p"=>8, "pau"=>8, "pcl"=>9, "q"=>8, "r"=>7, "s"=>12, "sh"=>12, "t"=>9, "tcl"=>10, "th"=>10, "uh"=>2, "uw"=>1, "ux"=>2, "v"=>10, "w"=>7, "y"=>9, "z"=>12, "zh"=>12, "DEL"=>5 },
  "aw" => { "aa"=>4, "ae"=>2, "ah"=>5, "ao"=>7, "aw"=>0, "ax"=>4, "ax-h"=>5, "axr"=>6, "ay"=>2, "b"=>9, "bcl"=>10, "ch"=>14, "d"=>10, "dcl"=>11, "dh"=>11, "dx"=>11, "eh"=>3, "el"=>10, "em"=>8, "en"=>9, "eng"=>9, "epi"=>9, "er"=>6, "ey"=>5, "f"=>13, "g"=>10, "gcl"=>11, "h#"=>9, "hh"=>9, "hv"=>8, "ih"=>4, "ix"=>3, "iy"=>5, "jh"=>13, "k"=>12, "kcl"=>13, "l"=>11, "m"=>9, "n"=>10, "ng"=>10, "nx"=>11, "ow"=>7, "oy"=>7, "p"=>11, "pau"=>9, "pcl"=>12, "q"=>11, "r"=>10, "s"=>15, "sh"=>15, "t"=>12, "tcl"=>13, "th"=>13, "uh"=>7, "uw"=>8, "ux"=>7, "v"=>11, "w"=>12, "y"=>10, "z"=>13, "zh"=>13, "DEL"=>5 },
  "ax" => { "aa"=>2, "ae"=>2, "ah"=>1, "ao"=>3, "aw"=>4, "ax"=>0, "ax-h"=>1, "axr"=>2, "ay"=>4, "b"=>5, "bcl"=>6, "ch"=>10, "d"=>6, "dcl"=>7, "dh"=>7, "dx"=>7, "eh"=>1, "el"=>6, "em"=>4, "en"=>5, "eng"=>5, "epi"=>5, "er"=>4, "ey"=>3, "f"=>9, "g"=>6, "gcl"=>7, "h#"=>5, "hh"=>7, "hv"=>6, "ih"=>2, "ix"=>1, "iy"=>3, "jh"=>9, "k"=>8, "kcl"=>9, "l"=>7, "m"=>5, "n"=>6, "ng"=>6, "nx"=>7, "ow"=>3, "oy"=>3, "p"=>7, "pau"=>5, "pcl"=>8, "q"=>7, "r"=>6, "s"=>11, "sh"=>11, "t"=>8, "tcl"=>9, "th"=>9, "uh"=>3, "uw"=>4, "ux"=>5, "v"=>7, "w"=>8, "y"=>6, "z"=>9, "zh"=>9, "DEL"=>5 },
  "ax-h" => { "aa"=>3, "ae"=>3, "ah"=>2, "ao"=>4, "aw"=>5, "ax"=>1, "ax-h"=>0, "axr"=>3, "ay"=>5, "b"=>6, "bcl"=>7, "ch"=>9, "d"=>7, "dcl"=>8, "dh"=>8, "dx"=>8, "eh"=>2, "el"=>7, "em"=>5, "en"=>6, "eng"=>6, "epi"=>4, "er"=>5, "ey"=>4, "f"=>8, "g"=>7, "gcl"=>8, "h#"=>4, "hh"=>6, "hv"=>7, "ih"=>3, "ix"=>2, "iy"=>4, "jh"=>10, "k"=>7, "kcl"=>8, "l"=>8, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>4, "oy"=>4, "p"=>6, "pau"=>4, "pcl"=>7, "q"=>6, "r"=>7, "s"=>10, "sh"=>10, "t"=>7, "tcl"=>8, "th"=>8, "uh"=>4, "uw"=>5, "ux"=>6, "v"=>8, "w"=>9, "y"=>7, "z"=>10, "zh"=>10, "DEL"=>5 },
  "axr" => { "aa"=>4, "ae"=>4, "ah"=>3, "ao"=>3, "aw"=>6, "ax"=>2, "ax-h"=>3, "axr"=>0, "ay"=>6, "b"=>7, "bcl"=>8, "ch"=>12, "d"=>8, "dcl"=>9, "dh"=>9, "dx"=>9, "eh"=>3, "el"=>8, "em"=>6, "en"=>7, "eng"=>7, "epi"=>7, "er"=>2, "ey"=>5, "f"=>11, "g"=>8, "gcl"=>9, "h#"=>7, "hh"=>9, "hv"=>8, "ih"=>4, "ix"=>3, "iy"=>5, "jh"=>11, "k"=>10, "kcl"=>11, "l"=>9, "m"=>7, "n"=>8, "ng"=>8, "nx"=>9, "ow"=>3, "oy"=>3, "p"=>9, "pau"=>7, "pcl"=>10, "q"=>9, "r"=>4, "s"=>13, "sh"=>13, "t"=>10, "tcl"=>11, "th"=>11, "uh"=>3, "uw"=>4, "ux"=>5, "v"=>9, "w"=>8, "y"=>8, "z"=>11, "zh"=>11, "DEL"=>5 },
  "ay" => { "aa"=>4, "ae"=>2, "ah"=>5, "ao"=>7, "aw"=>2, "ax"=>4, "ax-h"=>5, "axr"=>6, "ay"=>0, "b"=>9, "bcl"=>10, "ch"=>14, "d"=>10, "dcl"=>11, "dh"=>11, "dx"=>11, "eh"=>3, "el"=>10, "em"=>8, "en"=>9, "eng"=>9, "epi"=>9, "er"=>6, "ey"=>3, "f"=>13, "g"=>10, "gcl"=>11, "h#"=>9, "hh"=>9, "hv"=>8, "ih"=>4, "ix"=>3, "iy"=>5, "jh"=>13, "k"=>12, "kcl"=>13, "l"=>11, "m"=>9, "n"=>10, "ng"=>10, "nx"=>11, "ow"=>7, "oy"=>5, "p"=>11, "pau"=>9, "pcl"=>12, "q"=>11, "r"=>10, "s"=>15, "sh"=>15, "t"=>12, "tcl"=>13, "th"=>13, "uh"=>7, "uw"=>8, "ux"=>7, "v"=>11, "w"=>12, "y"=>10, "z"=>13, "zh"=>13, "DEL"=>5 },
  "b" => { "aa"=>7, "ae"=>7, "ah"=>6, "ao"=>8, "aw"=>9, "ax"=>5, "ax-h"=>6, "axr"=>7, "ay"=>9, "b"=>0, "bcl"=>1, "ch"=>7, "d"=>1, "dcl"=>2, "dh"=>2, "dx"=>2, "eh"=>6, "el"=>7, "em"=>3, "en"=>4, "eng"=>6, "epi"=>4, "er"=>9, "ey"=>8, "f"=>4, "g"=>3, "gcl"=>4, "h#"=>4, "hh"=>4, "hv"=>3, "ih"=>7, "ix"=>6, "iy"=>8, "jh"=>6, "k"=>5, "kcl"=>6, "l"=>6, "m"=>2, "n"=>3, "ng"=>5, "nx"=>4, "ow"=>8, "oy"=>8, "p"=>2, "pau"=>4, "pcl"=>3, "q"=>4, "r"=>7, "s"=>6, "sh"=>8, "t"=>3, "tcl"=>4, "th"=>4, "uh"=>8, "uw"=>9, "ux"=>10, "v"=>2, "w"=>7, "y"=>5, "z"=>4, "zh"=>6, "DEL"=>5 },
  "bcl" => { "aa"=>8, "ae"=>8, "ah"=>7, "ao"=>9, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>10, "b"=>1, "bcl"=>0, "ch"=>8, "d"=>2, "dcl"=>1, "dh"=>3, "dx"=>3, "eh"=>7, "el"=>8, "em"=>4, "en"=>5, "eng"=>7, "epi"=>3, "er"=>10, "ey"=>9, "f"=>5, "g"=>4, "gcl"=>3, "h#"=>3, "hh"=>5, "hv"=>4, "ih"=>8, "ix"=>7, "iy"=>9, "jh"=>7, "k"=>6, "kcl"=>5, "l"=>7, "m"=>3, "n"=>4, "ng"=>6, "nx"=>5, "ow"=>9, "oy"=>9, "p"=>3, "pau"=>3, "pcl"=>2, "q"=>3, "r"=>8, "s"=>7, "sh"=>9, "t"=>4, "tcl"=>3, "th"=>5, "uh"=>9, "uw"=>10, "ux"=>11, "v"=>3, "w"=>8, "y"=>6, "z"=>5, "zh"=>7, "DEL"=>5 },
  "ch" => { "aa"=>12, "ae"=>12, "ah"=>9, "ao"=>11, "aw"=>14, "ax"=>10, "ax-h"=>9, "axr"=>12, "ay"=>14, "b"=>7, "bcl"=>8, "ch"=>0, "d"=>6, "dcl"=>7, "dh"=>7, "dx"=>7, "eh"=>11, "el"=>10, "em"=>10, "en"=>9, "eng"=>9, "epi"=>7, "er"=>12, "ey"=>11, "f"=>5, "g"=>6, "gcl"=>7, "h#"=>7, "hh"=>7, "hv"=>8, "ih"=>10, "ix"=>11, "iy"=>9, "jh"=>1, "k"=>4, "kcl"=>5, "l"=>9, "m"=>9, "n"=>8, "ng"=>8, "nx"=>9, "ow"=>11, "oy"=>13, "p"=>5, "pau"=>7, "pcl"=>6, "q"=>7, "r"=>10, "s"=>3, "sh"=>1, "t"=>4, "tcl"=>5, "th"=>5, "uh"=>11, "uw"=>10, "ux"=>11, "v"=>7, "w"=>8, "y"=>6, "z"=>5, "zh"=>3, "DEL"=>5 },
  "d" => { "aa"=>8, "ae"=>8, "ah"=>7, "ao"=>9, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>10, "b"=>1, "bcl"=>2, "ch"=>6, "d"=>0, "dcl"=>1, "dh"=>1, "dx"=>1, "eh"=>7, "el"=>6, "em"=>4, "en"=>3, "eng"=>7, "epi"=>5, "er"=>10, "ey"=>9, "f"=>5, "g"=>4, "gcl"=>5, "h#"=>5, "hh"=>5, "hv"=>4, "ih"=>8, "ix"=>7, "iy"=>9, "jh"=>5, "k"=>6, "kcl"=>7, "l"=>5, "m"=>3, "n"=>2, "ng"=>6, "nx"=>3, "ow"=>9, "oy"=>9, "p"=>3, "pau"=>5, "pcl"=>4, "q"=>5, "r"=>6, "s"=>5, "sh"=>7, "t"=>2, "tcl"=>3, "th"=>3, "uh"=>9, "uw"=>10, "ux"=>11, "v"=>3, "w"=>8, "y"=>6, "z"=>3, "zh"=>5, "DEL"=>5 },
  "dcl" => { "aa"=>9, "ae"=>9, "ah"=>8, "ao"=>10, "aw"=>11, "ax"=>7, "ax-h"=>8, "axr"=>9, "ay"=>11, "b"=>2, "bcl"=>1, "ch"=>7, "d"=>1, "dcl"=>0, "dh"=>2, "dx"=>2, "eh"=>8, "el"=>7, "em"=>5, "en"=>4, "eng"=>8, "epi"=>4, "er"=>11, "ey"=>10, "f"=>6, "g"=>5, "gcl"=>4, "h#"=>4, "hh"=>6, "hv"=>5, "ih"=>9, "ix"=>8, "iy"=>10, "jh"=>6, "k"=>7, "kcl"=>6, "l"=>6, "m"=>4, "n"=>3, "ng"=>7, "nx"=>4, "ow"=>10, "oy"=>10, "p"=>4, "pau"=>4, "pcl"=>3, "q"=>4, "r"=>7, "s"=>6, "sh"=>8, "t"=>3, "tcl"=>2, "th"=>4, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>4, "w"=>9, "y"=>7, "z"=>4, "zh"=>6, "DEL"=>5 },
  "dh" => { "aa"=>9, "ae"=>9, "ah"=>8, "ao"=>10, "aw"=>11, "ax"=>7, "ax-h"=>8, "axr"=>9, "ay"=>11, "b"=>2, "bcl"=>3, "ch"=>7, "d"=>1, "dcl"=>2, "dh"=>0, "dx"=>2, "eh"=>8, "el"=>5, "em"=>5, "en"=>4, "eng"=>8, "epi"=>6, "er"=>11, "ey"=>10, "f"=>4, "g"=>5, "gcl"=>6, "h#"=>6, "hh"=>4, "hv"=>3, "ih"=>9, "ix"=>8, "iy"=>10, "jh"=>6, "k"=>7, "kcl"=>8, "l"=>4, "m"=>4, "n"=>3, "ng"=>7, "nx"=>4, "ow"=>10, "oy"=>10, "p"=>4, "pau"=>6, "pcl"=>5, "q"=>6, "r"=>5, "s"=>4, "sh"=>6, "t"=>3, "tcl"=>4, "th"=>2, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>2, "w"=>7, "y"=>5, "z"=>2, "zh"=>4, "DEL"=>5 },
  "dx" => { "aa"=>9, "ae"=>9, "ah"=>8, "ao"=>10, "aw"=>11, "ax"=>7, "ax-h"=>8, "axr"=>9, "ay"=>11, "b"=>2, "bcl"=>3, "ch"=>7, "d"=>1, "dcl"=>2, "dh"=>2, "dx"=>0, "eh"=>8, "el"=>7, "em"=>5, "en"=>4, "eng"=>8, "epi"=>6, "er"=>11, "ey"=>10, "f"=>6, "g"=>5, "gcl"=>6, "h#"=>6, "hh"=>6, "hv"=>5, "ih"=>9, "ix"=>8, "iy"=>10, "jh"=>6, "k"=>7, "kcl"=>8, "l"=>6, "m"=>4, "n"=>3, "ng"=>7, "nx"=>2, "ow"=>10, "oy"=>10, "p"=>4, "pau"=>6, "pcl"=>5, "q"=>6, "r"=>7, "s"=>6, "sh"=>8, "t"=>3, "tcl"=>4, "th"=>4, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>4, "w"=>9, "y"=>7, "z"=>4, "zh"=>6, "DEL"=>5 },
  "eh" => { "aa"=>3, "ae"=>1, "ah"=>2, "ao"=>4, "aw"=>3, "ax"=>1, "ax-h"=>2, "axr"=>3, "ay"=>3, "b"=>6, "bcl"=>7, "ch"=>11, "d"=>7, "dcl"=>8, "dh"=>8, "dx"=>8, "eh"=>0, "el"=>7, "em"=>5, "en"=>6, "eng"=>6, "epi"=>6, "er"=>5, "ey"=>2, "f"=>10, "g"=>7, "gcl"=>8, "h#"=>6, "hh"=>8, "hv"=>7, "ih"=>1, "ix"=>0, "iy"=>2, "jh"=>10, "k"=>9, "kcl"=>10, "l"=>8, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>4, "oy"=>4, "p"=>8, "pau"=>6, "pcl"=>9, "q"=>8, "r"=>7, "s"=>12, "sh"=>12, "t"=>9, "tcl"=>10, "th"=>10, "uh"=>4, "uw"=>5, "ux"=>6, "v"=>8, "w"=>9, "y"=>7, "z"=>10, "zh"=>10, "DEL"=>5 },
  "el" => { "aa"=>8, "ae"=>8, "ah"=>5, "ao"=>7, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>10, "b"=>7, "bcl"=>8, "ch"=>10, "d"=>6, "dcl"=>7, "dh"=>5, "dx"=>7, "eh"=>7, "el"=>0, "em"=>6, "en"=>5, "eng"=>9, "epi"=>11, "er"=>8, "ey"=>7, "f"=>7, "g"=>10, "gcl"=>11, "h#"=>11, "hh"=>9, "hv"=>8, "ih"=>8, "ix"=>7, "iy"=>7, "jh"=>9, "k"=>10, "kcl"=>11, "l"=>1, "m"=>7, "n"=>6, "ng"=>10, "nx"=>7, "ow"=>7, "oy"=>9, "p"=>7, "pau"=>11, "pcl"=>8, "q"=>11, "r"=>6, "s"=>7, "sh"=>9, "t"=>6, "tcl"=>7, "th"=>5, "uh"=>9, "uw"=>8, "ux"=>9, "v"=>7, "w"=>10, "y"=>8, "z"=>7, "zh"=>9, "DEL"=>5 },
  "em" => { "aa"=>6, "ae"=>6, "ah"=>5, "ao"=>7, "aw"=>8, "ax"=>4, "ax-h"=>5, "axr"=>6, "ay"=>8, "b"=>3, "bcl"=>4, "ch"=>10, "d"=>4, "dcl"=>5, "dh"=>5, "dx"=>5, "eh"=>5, "el"=>6, "em"=>0, "en"=>1, "eng"=>3, "epi"=>7, "er"=>8, "ey"=>7, "f"=>7, "g"=>6, "gcl"=>7, "h#"=>7, "hh"=>7, "hv"=>6, "ih"=>6, "ix"=>5, "iy"=>7, "jh"=>9, "k"=>8, "kcl"=>9, "l"=>7, "m"=>1, "n"=>2, "ng"=>4, "nx"=>3, "ow"=>7, "oy"=>7, "p"=>5, "pau"=>7, "pcl"=>6, "q"=>7, "r"=>8, "s"=>9, "sh"=>11, "t"=>6, "tcl"=>7, "th"=>7, "uh"=>7, "uw"=>8, "ux"=>9, "v"=>5, "w"=>8, "y"=>6, "z"=>7, "zh"=>9, "DEL"=>5 },
  "en" => { "aa"=>7, "ae"=>7, "ah"=>6, "ao"=>8, "aw"=>9, "ax"=>5, "ax-h"=>6, "axr"=>7, "ay"=>9, "b"=>4, "bcl"=>5, "ch"=>9, "d"=>3, "dcl"=>4, "dh"=>4, "dx"=>4, "eh"=>6, "el"=>5, "em"=>1, "en"=>0, "eng"=>4, "epi"=>8, "er"=>9, "ey"=>8, "f"=>8, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>8, "hv"=>7, "ih"=>7, "ix"=>6, "iy"=>8, "jh"=>8, "k"=>9, "kcl"=>10, "l"=>6, "m"=>2, "n"=>1, "ng"=>5, "nx"=>2, "ow"=>8, "oy"=>8, "p"=>6, "pau"=>8, "pcl"=>7, "q"=>8, "r"=>7, "s"=>8, "sh"=>10, "t"=>5, "tcl"=>6, "th"=>6, "uh"=>8, "uw"=>9, "ux"=>10, "v"=>6, "w"=>9, "y"=>7, "z"=>6, "zh"=>8, "DEL"=>5 },
  "eng" => { "aa"=>5, "ae"=>7, "ah"=>6, "ao"=>6, "aw"=>9, "ax"=>5, "ax-h"=>6, "axr"=>7, "ay"=>9, "b"=>6, "bcl"=>7, "ch"=>9, "d"=>7, "dcl"=>8, "dh"=>8, "dx"=>8, "eh"=>6, "el"=>9, "em"=>3, "en"=>4, "eng"=>0, "epi"=>8, "er"=>9, "ey"=>8, "f"=>10, "g"=>3, "gcl"=>4, "h#"=>8, "hh"=>8, "hv"=>7, "ih"=>5, "ix"=>6, "iy"=>6, "jh"=>8, "k"=>5, "kcl"=>6, "l"=>10, "m"=>4, "n"=>5, "ng"=>1, "nx"=>6, "ow"=>6, "oy"=>6, "p"=>8, "pau"=>8, "pcl"=>9, "q"=>6, "r"=>9, "s"=>10, "sh"=>10, "t"=>9, "tcl"=>10, "th"=>10, "uh"=>4, "uw"=>5, "ux"=>6, "v"=>8, "w"=>5, "y"=>5, "z"=>8, "zh"=>8, "DEL"=>5 },
  "epi" => { "aa"=>7, "ae"=>7, "ah"=>6, "ao"=>8, "aw"=>9, "ax"=>5, "ax-h"=>4, "axr"=>7, "ay"=>9, "b"=>4, "bcl"=>3, "ch"=>7, "d"=>5, "dcl"=>4, "dh"=>6, "dx"=>6, "eh"=>6, "el"=>11, "em"=>7, "en"=>8, "eng"=>8, "epi"=>0, "er"=>9, "ey"=>8, "f"=>6, "g"=>5, "gcl"=>4, "h#"=>0, "hh"=>4, "hv"=>5, "ih"=>7, "ix"=>6, "iy"=>8, "jh"=>8, "k"=>5, "kcl"=>4, "l"=>10, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>8, "oy"=>8, "p"=>4, "pau"=>0, "pcl"=>3, "q"=>2, "r"=>9, "s"=>8, "sh"=>8, "t"=>5, "tcl"=>4, "th"=>6, "uh"=>8, "uw"=>9, "ux"=>10, "v"=>6, "w"=>9, "y"=>7, "z"=>8, "zh"=>8, "DEL"=>5 },
  "er" => { "aa"=>6, "ae"=>6, "ah"=>3, "ao"=>3, "aw"=>6, "ax"=>4, "ax-h"=>5, "axr"=>2, "ay"=>6, "b"=>9, "bcl"=>10, "ch"=>12, "d"=>10, "dcl"=>11, "dh"=>11, "dx"=>11, "eh"=>5, "el"=>8, "em"=>8, "en"=>9, "eng"=>9, "epi"=>9, "er"=>0, "ey"=>5, "f"=>11, "g"=>10, "gcl"=>11, "h#"=>9, "hh"=>11, "hv"=>10, "ih"=>6, "ix"=>5, "iy"=>5, "jh"=>11, "k"=>10, "kcl"=>11, "l"=>9, "m"=>9, "n"=>10, "ng"=>10, "nx"=>11, "ow"=>3, "oy"=>5, "p"=>9, "pau"=>9, "pcl"=>10, "q"=>11, "r"=>6, "s"=>13, "sh"=>13, "t"=>10, "tcl"=>11, "th"=>11, "uh"=>5, "uw"=>4, "ux"=>3, "v"=>11, "w"=>10, "y"=>10, "z"=>13, "zh"=>13, "DEL"=>5 },
  "ey" => { "aa"=>5, "ae"=>3, "ah"=>2, "ao"=>4, "aw"=>5, "ax"=>3, "ax-h"=>4, "axr"=>5, "ay"=>3, "b"=>8, "bcl"=>9, "ch"=>11, "d"=>9, "dcl"=>10, "dh"=>10, "dx"=>10, "eh"=>2, "el"=>7, "em"=>7, "en"=>8, "eng"=>8, "epi"=>8, "er"=>5, "ey"=>0, "f"=>10, "g"=>9, "gcl"=>10, "h#"=>8, "hh"=>10, "hv"=>9, "ih"=>3, "ix"=>2, "iy"=>2, "jh"=>10, "k"=>9, "kcl"=>10, "l"=>8, "m"=>8, "n"=>9, "ng"=>9, "nx"=>10, "ow"=>4, "oy"=>4, "p"=>8, "pau"=>8, "pcl"=>9, "q"=>10, "r"=>9, "s"=>12, "sh"=>12, "t"=>9, "tcl"=>10, "th"=>10, "uh"=>6, "uw"=>5, "ux"=>6, "v"=>10, "w"=>11, "y"=>9, "z"=>12, "zh"=>12, "DEL"=>5 },
  "f" => { "aa"=>11, "ae"=>11, "ah"=>8, "ao"=>10, "aw"=>13, "ax"=>9, "ax-h"=>8, "axr"=>11, "ay"=>13, "b"=>4, "bcl"=>5, "ch"=>5, "d"=>5, "dcl"=>6, "dh"=>4, "dx"=>6, "eh"=>10, "el"=>7, "em"=>7, "en"=>8, "eng"=>10, "epi"=>6, "er"=>11, "ey"=>10, "f"=>0, "g"=>7, "gcl"=>8, "h#"=>6, "hh"=>4, "hv"=>5, "ih"=>11, "ix"=>10, "iy"=>10, "jh"=>6, "k"=>5, "kcl"=>6, "l"=>6, "m"=>6, "n"=>7, "ng"=>9, "nx"=>8, "ow"=>10, "oy"=>12, "p"=>2, "pau"=>6, "pcl"=>3, "q"=>6, "r"=>9, "s"=>2, "sh"=>4, "t"=>3, "tcl"=>4, "th"=>2, "uh"=>12, "uw"=>11, "ux"=>12, "v"=>2, "w"=>9, "y"=>7, "z"=>4, "zh"=>6, "DEL"=>5 },
  "g" => { "aa"=>6, "ae"=>8, "ah"=>7, "ao"=>7, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>10, "b"=>3, "bcl"=>4, "ch"=>6, "d"=>4, "dcl"=>5, "dh"=>5, "dx"=>5, "eh"=>7, "el"=>10, "em"=>6, "en"=>7, "eng"=>3, "epi"=>5, "er"=>10, "ey"=>9, "f"=>7, "g"=>0, "gcl"=>1, "h#"=>5, "hh"=>5, "hv"=>4, "ih"=>6, "ix"=>7, "iy"=>7, "jh"=>5, "k"=>2, "kcl"=>3, "l"=>9, "m"=>5, "n"=>6, "ng"=>2, "nx"=>7, "ow"=>7, "oy"=>7, "p"=>5, "pau"=>5, "pcl"=>6, "q"=>3, "r"=>8, "s"=>7, "sh"=>7, "t"=>6, "tcl"=>7, "th"=>7, "uh"=>5, "uw"=>6, "ux"=>7, "v"=>5, "w"=>4, "y"=>4, "z"=>5, "zh"=>5, "DEL"=>5 },
  "gcl" => { "aa"=>7, "ae"=>9, "ah"=>8, "ao"=>8, "aw"=>11, "ax"=>7, "ax-h"=>8, "axr"=>9, "ay"=>11, "b"=>4, "bcl"=>3, "ch"=>7, "d"=>5, "dcl"=>4, "dh"=>6, "dx"=>6, "eh"=>8, "el"=>11, "em"=>7, "en"=>8, "eng"=>4, "epi"=>4, "er"=>11, "ey"=>10, "f"=>8, "g"=>1, "gcl"=>0, "h#"=>4, "hh"=>6, "hv"=>5, "ih"=>7, "ix"=>8, "iy"=>8, "jh"=>6, "k"=>3, "kcl"=>2, "l"=>10, "m"=>6, "n"=>7, "ng"=>3, "nx"=>8, "ow"=>8, "oy"=>8, "p"=>6, "pau"=>4, "pcl"=>5, "q"=>2, "r"=>9, "s"=>8, "sh"=>8, "t"=>7, "tcl"=>6, "th"=>8, "uh"=>6, "uw"=>7, "ux"=>8, "v"=>6, "w"=>5, "y"=>5, "z"=>6, "zh"=>6, "DEL"=>5 },
  "h#" => { "aa"=>7, "ae"=>7, "ah"=>6, "ao"=>8, "aw"=>9, "ax"=>5, "ax-h"=>4, "axr"=>7, "ay"=>9, "b"=>4, "bcl"=>3, "ch"=>7, "d"=>5, "dcl"=>4, "dh"=>6, "dx"=>6, "eh"=>6, "el"=>11, "em"=>7, "en"=>8, "eng"=>8, "epi"=>0, "er"=>9, "ey"=>8, "f"=>6, "g"=>5, "gcl"=>4, "h#"=>0, "hh"=>4, "hv"=>5, "ih"=>7, "ix"=>6, "iy"=>8, "jh"=>8, "k"=>5, "kcl"=>4, "l"=>10, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>8, "oy"=>8, "p"=>4, "pau"=>0, "pcl"=>3, "q"=>2, "r"=>9, "s"=>8, "sh"=>8, "t"=>5, "tcl"=>4, "th"=>6, "uh"=>8, "uw"=>9, "ux"=>10, "v"=>6, "w"=>9, "y"=>7, "z"=>8, "zh"=>8, "DEL"=>5 },
  "hh" => { "aa"=>7, "ae"=>7, "ah"=>8, "ao"=>10, "aw"=>9, "ax"=>7, "ax-h"=>6, "axr"=>9, "ay"=>9, "b"=>4, "bcl"=>5, "ch"=>7, "d"=>5, "dcl"=>6, "dh"=>4, "dx"=>6, "eh"=>8, "el"=>9, "em"=>7, "en"=>8, "eng"=>8, "epi"=>4, "er"=>11, "ey"=>10, "f"=>4, "g"=>5, "gcl"=>6, "h#"=>4, "hh"=>0, "hv"=>1, "ih"=>9, "ix"=>8, "iy"=>10, "jh"=>8, "k"=>5, "kcl"=>6, "l"=>8, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>10, "oy"=>10, "p"=>4, "pau"=>4, "pcl"=>5, "q"=>4, "r"=>7, "s"=>6, "sh"=>6, "t"=>5, "tcl"=>6, "th"=>4, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>4, "w"=>7, "y"=>5, "z"=>6, "zh"=>6, "DEL"=>5 },
  "hv" => { "aa"=>6, "ae"=>6, "ah"=>7, "ao"=>9, "aw"=>8, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>8, "b"=>3, "bcl"=>4, "ch"=>8, "d"=>4, "dcl"=>5, "dh"=>3, "dx"=>5, "eh"=>7, "el"=>8, "em"=>6, "en"=>7, "eng"=>7, "epi"=>5, "er"=>10, "ey"=>9, "f"=>5, "g"=>4, "gcl"=>5, "h#"=>5, "hh"=>1, "hv"=>0, "ih"=>8, "ix"=>7, "iy"=>9, "jh"=>7, "k"=>6, "kcl"=>7, "l"=>7, "m"=>5, "n"=>6, "ng"=>6, "nx"=>7, "ow"=>9, "oy"=>9, "p"=>5, "pau"=>5, "pcl"=>6, "q"=>5, "r"=>6, "s"=>7, "sh"=>7, "t"=>6, "tcl"=>7, "th"=>5, "uh"=>9, "uw"=>10, "ux"=>11, "v"=>3, "w"=>6, "y"=>4, "z"=>5, "zh"=>5, "DEL"=>5 },
  "ih" => { "aa"=>4, "ae"=>2, "ah"=>3, "ao"=>5, "aw"=>4, "ax"=>2, "ax-h"=>3, "axr"=>4, "ay"=>4, "b"=>7, "bcl"=>8, "ch"=>10, "d"=>8, "dcl"=>9, "dh"=>9, "dx"=>9, "eh"=>1, "el"=>8, "em"=>6, "en"=>7, "eng"=>5, "epi"=>7, "er"=>6, "ey"=>3, "f"=>11, "g"=>6, "gcl"=>7, "h#"=>7, "hh"=>9, "hv"=>8, "ih"=>0, "ix"=>1, "iy"=>1, "jh"=>9, "k"=>8, "kcl"=>9, "l"=>9, "m"=>7, "n"=>8, "ng"=>6, "nx"=>9, "ow"=>5, "oy"=>5, "p"=>9, "pau"=>7, "pcl"=>10, "q"=>9, "r"=>8, "s"=>11, "sh"=>11, "t"=>10, "tcl"=>11, "th"=>11, "uh"=>3, "uw"=>4, "ux"=>5, "v"=>9, "w"=>8, "y"=>6, "z"=>9, "zh"=>9, "DEL"=>5 },
  "ix" => { "aa"=>3, "ae"=>1, "ah"=>2, "ao"=>4, "aw"=>3, "ax"=>1, "ax-h"=>2, "axr"=>3, "ay"=>3, "b"=>6, "bcl"=>7, "ch"=>11, "d"=>7, "dcl"=>8, "dh"=>8, "dx"=>8, "eh"=>0, "el"=>7, "em"=>5, "en"=>6, "eng"=>6, "epi"=>6, "er"=>5, "ey"=>2, "f"=>10, "g"=>7, "gcl"=>8, "h#"=>6, "hh"=>8, "hv"=>7, "ih"=>1, "ix"=>0, "iy"=>2, "jh"=>10, "k"=>9, "kcl"=>10, "l"=>8, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>4, "oy"=>4, "p"=>8, "pau"=>6, "pcl"=>9, "q"=>8, "r"=>7, "s"=>12, "sh"=>12, "t"=>9, "tcl"=>10, "th"=>10, "uh"=>4, "uw"=>5, "ux"=>6, "v"=>8, "w"=>9, "y"=>7, "z"=>10, "zh"=>10, "DEL"=>5 },
  "iy" => { "aa"=>5, "ae"=>3, "ah"=>2, "ao"=>4, "aw"=>5, "ax"=>3, "ax-h"=>4, "axr"=>5, "ay"=>5, "b"=>8, "bcl"=>9, "ch"=>9, "d"=>9, "dcl"=>10, "dh"=>10, "dx"=>10, "eh"=>2, "el"=>7, "em"=>7, "en"=>8, "eng"=>6, "epi"=>8, "er"=>5, "ey"=>2, "f"=>10, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>10, "hv"=>9, "ih"=>1, "ix"=>2, "iy"=>0, "jh"=>8, "k"=>7, "kcl"=>8, "l"=>8, "m"=>8, "n"=>9, "ng"=>7, "nx"=>10, "ow"=>4, "oy"=>6, "p"=>8, "pau"=>8, "pcl"=>9, "q"=>10, "r"=>9, "s"=>10, "sh"=>10, "t"=>9, "tcl"=>10, "th"=>10, "uh"=>4, "uw"=>3, "ux"=>4, "v"=>10, "w"=>9, "y"=>7, "z"=>10, "zh"=>10, "DEL"=>5 },
  "jh" => { "aa"=>11, "ae"=>11, "ah"=>8, "ao"=>10, "aw"=>13, "ax"=>9, "ax-h"=>10, "axr"=>11, "ay"=>13, "b"=>6, "bcl"=>7, "ch"=>1, "d"=>5, "dcl"=>6, "dh"=>6, "dx"=>6, "eh"=>10, "el"=>9, "em"=>9, "en"=>8, "eng"=>8, "epi"=>8, "er"=>11, "ey"=>10, "f"=>6, "g"=>5, "gcl"=>6, "h#"=>8, "hh"=>8, "hv"=>7, "ih"=>9, "ix"=>10, "iy"=>8, "jh"=>0, "k"=>5, "kcl"=>6, "l"=>8, "m"=>8, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>10, "oy"=>12, "p"=>6, "pau"=>8, "pcl"=>7, "q"=>8, "r"=>9, "s"=>4, "sh"=>2, "t"=>5, "tcl"=>6, "th"=>6, "uh"=>10, "uw"=>9, "ux"=>10, "v"=>6, "w"=>7, "y"=>5, "z"=>4, "zh"=>2, "DEL"=>5 },
  "k" => { "aa"=>8, "ae"=>10, "ah"=>7, "ao"=>7, "aw"=>12, "ax"=>8, "ax-h"=>7, "axr"=>10, "ay"=>12, "b"=>5, "bcl"=>6, "ch"=>4, "d"=>6, "dcl"=>7, "dh"=>7, "dx"=>7, "eh"=>9, "el"=>10, "em"=>8, "en"=>9, "eng"=>5, "epi"=>5, "er"=>10, "ey"=>9, "f"=>5, "g"=>2, "gcl"=>3, "h#"=>5, "hh"=>5, "hv"=>6, "ih"=>8, "ix"=>9, "iy"=>7, "jh"=>5, "k"=>0, "kcl"=>1, "l"=>9, "m"=>7, "n"=>8, "ng"=>4, "nx"=>9, "ow"=>7, "oy"=>9, "p"=>3, "pau"=>5, "pcl"=>4, "q"=>3, "r"=>10, "s"=>5, "sh"=>5, "t"=>4, "tcl"=>5, "th"=>5, "uh"=>7, "uw"=>6, "ux"=>7, "v"=>7, "w"=>6, "y"=>6, "z"=>7, "zh"=>7, "DEL"=>5 },
  "kcl" => { "aa"=>9, "ae"=>11, "ah"=>8, "ao"=>8, "aw"=>13, "ax"=>9, "ax-h"=>8, "axr"=>11, "ay"=>13, "b"=>6, "bcl"=>5, "ch"=>5, "d"=>7, "dcl"=>6, "dh"=>8, "dx"=>8, "eh"=>10, "el"=>11, "em"=>9, "en"=>10, "eng"=>6, "epi"=>4, "er"=>11, "ey"=>10, "f"=>6, "g"=>3, "gcl"=>2, "h#"=>4, "hh"=>6, "hv"=>7, "ih"=>9, "ix"=>10, "iy"=>8, "jh"=>6, "k"=>1, "kcl"=>0, "l"=>10, "m"=>8, "n"=>9, "ng"=>5, "nx"=>10, "ow"=>8, "oy"=>10, "p"=>4, "pau"=>4, "pcl"=>3, "q"=>2, "r"=>11, "s"=>6, "sh"=>6, "t"=>5, "tcl"=>4, "th"=>6, "uh"=>8, "uw"=>7, "ux"=>8, "v"=>8, "w"=>7, "y"=>7, "z"=>8, "zh"=>8, "DEL"=>5 },
  "l" => { "aa"=>9, "ae"=>9, "ah"=>6, "ao"=>8, "aw"=>11, "ax"=>7, "ax-h"=>8, "axr"=>9, "ay"=>11, "b"=>6, "bcl"=>7, "ch"=>9, "d"=>5, "dcl"=>6, "dh"=>4, "dx"=>6, "eh"=>8, "el"=>1, "em"=>7, "en"=>6, "eng"=>10, "epi"=>10, "er"=>9, "ey"=>8, "f"=>6, "g"=>9, "gcl"=>10, "h#"=>10, "hh"=>8, "hv"=>7, "ih"=>9, "ix"=>8, "iy"=>8, "jh"=>8, "k"=>9, "kcl"=>10, "l"=>0, "m"=>6, "n"=>5, "ng"=>9, "nx"=>6, "ow"=>8, "oy"=>10, "p"=>6, "pau"=>10, "pcl"=>7, "q"=>10, "r"=>5, "s"=>6, "sh"=>8, "t"=>5, "tcl"=>6, "th"=>4, "uh"=>10, "uw"=>9, "ux"=>10, "v"=>6, "w"=>9, "y"=>7, "z"=>6, "zh"=>8, "DEL"=>5 },
  "m" => { "aa"=>7, "ae"=>7, "ah"=>6, "ao"=>8, "aw"=>9, "ax"=>5, "ax-h"=>6, "axr"=>7, "ay"=>9, "b"=>2, "bcl"=>3, "ch"=>9, "d"=>3, "dcl"=>4, "dh"=>4, "dx"=>4, "eh"=>6, "el"=>7, "em"=>1, "en"=>2, "eng"=>4, "epi"=>6, "er"=>9, "ey"=>8, "f"=>6, "g"=>5, "gcl"=>6, "h#"=>6, "hh"=>6, "hv"=>5, "ih"=>7, "ix"=>6, "iy"=>8, "jh"=>8, "k"=>7, "kcl"=>8, "l"=>6, "m"=>0, "n"=>1, "ng"=>3, "nx"=>2, "ow"=>8, "oy"=>8, "p"=>4, "pau"=>6, "pcl"=>5, "q"=>6, "r"=>7, "s"=>8, "sh"=>10, "t"=>5, "tcl"=>6, "th"=>6, "uh"=>8, "uw"=>9, "ux"=>10, "v"=>4, "w"=>7, "y"=>5, "z"=>6, "zh"=>8, "DEL"=>5 },
  "n" => { "aa"=>8, "ae"=>8, "ah"=>7, "ao"=>9, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>10, "b"=>3, "bcl"=>4, "ch"=>8, "d"=>2, "dcl"=>3, "dh"=>3, "dx"=>3, "eh"=>7, "el"=>6, "em"=>2, "en"=>1, "eng"=>5, "epi"=>7, "er"=>10, "ey"=>9, "f"=>7, "g"=>6, "gcl"=>7, "h#"=>7, "hh"=>7, "hv"=>6, "ih"=>8, "ix"=>7, "iy"=>9, "jh"=>7, "k"=>8, "kcl"=>9, "l"=>5, "m"=>1, "n"=>0, "ng"=>4, "nx"=>1, "ow"=>9, "oy"=>9, "p"=>5, "pau"=>7, "pcl"=>6, "q"=>7, "r"=>6, "s"=>7, "sh"=>9, "t"=>4, "tcl"=>5, "th"=>5, "uh"=>9, "uw"=>10, "ux"=>11, "v"=>5, "w"=>8, "y"=>6, "z"=>5, "zh"=>7, "DEL"=>5 },
  "ng" => { "aa"=>6, "ae"=>8, "ah"=>7, "ao"=>7, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>10, "b"=>5, "bcl"=>6, "ch"=>8, "d"=>6, "dcl"=>7, "dh"=>7, "dx"=>7, "eh"=>7, "el"=>10, "em"=>4, "en"=>5, "eng"=>1, "epi"=>7, "er"=>10, "ey"=>9, "f"=>9, "g"=>2, "gcl"=>3, "h#"=>7, "hh"=>7, "hv"=>6, "ih"=>6, "ix"=>7, "iy"=>7, "jh"=>7, "k"=>4, "kcl"=>5, "l"=>9, "m"=>3, "n"=>4, "ng"=>0, "nx"=>5, "ow"=>7, "oy"=>7, "p"=>7, "pau"=>7, "pcl"=>8, "q"=>5, "r"=>8, "s"=>9, "sh"=>9, "t"=>8, "tcl"=>9, "th"=>9, "uh"=>5, "uw"=>6, "ux"=>7, "v"=>7, "w"=>4, "y"=>4, "z"=>7, "zh"=>7, "DEL"=>5 },
  "nx" => { "aa"=>9, "ae"=>9, "ah"=>8, "ao"=>10, "aw"=>11, "ax"=>7, "ax-h"=>8, "axr"=>9, "ay"=>11, "b"=>4, "bcl"=>5, "ch"=>9, "d"=>3, "dcl"=>4, "dh"=>4, "dx"=>2, "eh"=>8, "el"=>7, "em"=>3, "en"=>2, "eng"=>6, "epi"=>8, "er"=>11, "ey"=>10, "f"=>8, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>8, "hv"=>7, "ih"=>9, "ix"=>8, "iy"=>10, "jh"=>8, "k"=>9, "kcl"=>10, "l"=>6, "m"=>2, "n"=>1, "ng"=>5, "nx"=>0, "ow"=>10, "oy"=>10, "p"=>6, "pau"=>8, "pcl"=>7, "q"=>8, "r"=>7, "s"=>8, "sh"=>10, "t"=>5, "tcl"=>6, "th"=>6, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>6, "w"=>9, "y"=>7, "z"=>6, "zh"=>8, "DEL"=>5 },
  "ow" => { "aa"=>3, "ae"=>5, "ah"=>2, "ao"=>0, "aw"=>7, "ax"=>3, "ax-h"=>4, "axr"=>3, "ay"=>7, "b"=>8, "bcl"=>9, "ch"=>11, "d"=>9, "dcl"=>10, "dh"=>10, "dx"=>10, "eh"=>4, "el"=>7, "em"=>7, "en"=>8, "eng"=>6, "epi"=>8, "er"=>3, "ey"=>4, "f"=>10, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>10, "hv"=>9, "ih"=>5, "ix"=>4, "iy"=>4, "jh"=>10, "k"=>7, "kcl"=>8, "l"=>8, "m"=>8, "n"=>9, "ng"=>7, "nx"=>10, "ow"=>0, "oy"=>2, "p"=>8, "pau"=>8, "pcl"=>9, "q"=>8, "r"=>7, "s"=>12, "sh"=>12, "t"=>9, "tcl"=>10, "th"=>10, "uh"=>2, "uw"=>1, "ux"=>2, "v"=>10, "w"=>7, "y"=>9, "z"=>12, "zh"=>12, "DEL"=>5 },
  "oy" => { "aa"=>3, "ae"=>5, "ah"=>4, "ao"=>2, "aw"=>7, "ax"=>3, "ax-h"=>4, "axr"=>3, "ay"=>5, "b"=>8, "bcl"=>9, "ch"=>13, "d"=>9, "dcl"=>10, "dh"=>10, "dx"=>10, "eh"=>4, "el"=>9, "em"=>7, "en"=>8, "eng"=>6, "epi"=>8, "er"=>5, "ey"=>4, "f"=>12, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>10, "hv"=>9, "ih"=>5, "ix"=>4, "iy"=>6, "jh"=>12, "k"=>9, "kcl"=>10, "l"=>10, "m"=>8, "n"=>9, "ng"=>7, "nx"=>10, "ow"=>2, "oy"=>0, "p"=>10, "pau"=>8, "pcl"=>11, "q"=>8, "r"=>7, "s"=>14, "sh"=>14, "t"=>11, "tcl"=>12, "th"=>12, "uh"=>2, "uw"=>3, "ux"=>4, "v"=>10, "w"=>7, "y"=>9, "z"=>12, "zh"=>12, "DEL"=>5 },
  "p" => { "aa"=>9, "ae"=>9, "ah"=>6, "ao"=>8, "aw"=>11, "ax"=>7, "ax-h"=>6, "axr"=>9, "ay"=>11, "b"=>2, "bcl"=>3, "ch"=>5, "d"=>3, "dcl"=>4, "dh"=>4, "dx"=>4, "eh"=>8, "el"=>7, "em"=>5, "en"=>6, "eng"=>8, "epi"=>4, "er"=>9, "ey"=>8, "f"=>2, "g"=>5, "gcl"=>6, "h#"=>4, "hh"=>4, "hv"=>5, "ih"=>9, "ix"=>8, "iy"=>8, "jh"=>6, "k"=>3, "kcl"=>4, "l"=>6, "m"=>4, "n"=>5, "ng"=>7, "nx"=>6, "ow"=>8, "oy"=>10, "p"=>0, "pau"=>4, "pcl"=>1, "q"=>4, "r"=>9, "s"=>4, "sh"=>6, "t"=>1, "tcl"=>2, "th"=>2, "uh"=>10, "uw"=>9, "ux"=>10, "v"=>4, "w"=>9, "y"=>7, "z"=>6, "zh"=>8, "DEL"=>5 },
  "pau" => { "aa"=>7, "ae"=>7, "ah"=>6, "ao"=>8, "aw"=>9, "ax"=>5, "ax-h"=>4, "axr"=>7, "ay"=>9, "b"=>4, "bcl"=>3, "ch"=>7, "d"=>5, "dcl"=>4, "dh"=>6, "dx"=>6, "eh"=>6, "el"=>11, "em"=>7, "en"=>8, "eng"=>8, "epi"=>0, "er"=>9, "ey"=>8, "f"=>6, "g"=>5, "gcl"=>4, "h#"=>0, "hh"=>4, "hv"=>5, "ih"=>7, "ix"=>6, "iy"=>8, "jh"=>8, "k"=>5, "kcl"=>4, "l"=>10, "m"=>6, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>8, "oy"=>8, "p"=>4, "pau"=>0, "pcl"=>3, "q"=>2, "r"=>9, "s"=>8, "sh"=>8, "t"=>5, "tcl"=>4, "th"=>6, "uh"=>8, "uw"=>9, "ux"=>10, "v"=>6, "w"=>9, "y"=>7, "z"=>8, "zh"=>8, "DEL"=>5 },
  "pcl" => { "aa"=>10, "ae"=>10, "ah"=>7, "ao"=>9, "aw"=>12, "ax"=>8, "ax-h"=>7, "axr"=>10, "ay"=>12, "b"=>3, "bcl"=>2, "ch"=>6, "d"=>4, "dcl"=>3, "dh"=>5, "dx"=>5, "eh"=>9, "el"=>8, "em"=>6, "en"=>7, "eng"=>9, "epi"=>3, "er"=>10, "ey"=>9, "f"=>3, "g"=>6, "gcl"=>5, "h#"=>3, "hh"=>5, "hv"=>6, "ih"=>10, "ix"=>9, "iy"=>9, "jh"=>7, "k"=>4, "kcl"=>3, "l"=>7, "m"=>5, "n"=>6, "ng"=>8, "nx"=>7, "ow"=>9, "oy"=>11, "p"=>1, "pau"=>3, "pcl"=>0, "q"=>3, "r"=>10, "s"=>5, "sh"=>7, "t"=>2, "tcl"=>1, "th"=>3, "uh"=>11, "uw"=>10, "ux"=>11, "v"=>5, "w"=>10, "y"=>8, "z"=>7, "zh"=>9, "DEL"=>5 },
  "q" => { "aa"=>7, "ae"=>9, "ah"=>8, "ao"=>8, "aw"=>11, "ax"=>7, "ax-h"=>6, "axr"=>9, "ay"=>11, "b"=>4, "bcl"=>3, "ch"=>7, "d"=>5, "dcl"=>4, "dh"=>6, "dx"=>6, "eh"=>8, "el"=>11, "em"=>7, "en"=>8, "eng"=>6, "epi"=>2, "er"=>11, "ey"=>10, "f"=>6, "g"=>3, "gcl"=>2, "h#"=>2, "hh"=>4, "hv"=>5, "ih"=>9, "ix"=>8, "iy"=>10, "jh"=>8, "k"=>3, "kcl"=>2, "l"=>10, "m"=>6, "n"=>7, "ng"=>5, "nx"=>8, "ow"=>8, "oy"=>8, "p"=>4, "pau"=>2, "pcl"=>3, "q"=>0, "r"=>9, "s"=>8, "sh"=>8, "t"=>5, "tcl"=>4, "th"=>6, "uh"=>8, "uw"=>9, "ux"=>10, "v"=>6, "w"=>7, "y"=>7, "z"=>8, "zh"=>8, "DEL"=>5 },
  "r" => { "aa"=>8, "ae"=>8, "ah"=>7, "ao"=>7, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>4, "ay"=>10, "b"=>7, "bcl"=>8, "ch"=>10, "d"=>6, "dcl"=>7, "dh"=>5, "dx"=>7, "eh"=>7, "el"=>6, "em"=>8, "en"=>7, "eng"=>9, "epi"=>9, "er"=>6, "ey"=>9, "f"=>9, "g"=>8, "gcl"=>9, "h#"=>9, "hh"=>7, "hv"=>6, "ih"=>8, "ix"=>7, "iy"=>9, "jh"=>9, "k"=>10, "kcl"=>11, "l"=>5, "m"=>7, "n"=>6, "ng"=>8, "nx"=>7, "ow"=>7, "oy"=>7, "p"=>9, "pau"=>9, "pcl"=>10, "q"=>9, "r"=>0, "s"=>9, "sh"=>9, "t"=>8, "tcl"=>9, "th"=>7, "uh"=>7, "uw"=>8, "ux"=>9, "v"=>7, "w"=>6, "y"=>6, "z"=>7, "zh"=>7, "DEL"=>5 },
  "s" => { "aa"=>13, "ae"=>13, "ah"=>10, "ao"=>12, "aw"=>15, "ax"=>11, "ax-h"=>10, "axr"=>13, "ay"=>15, "b"=>6, "bcl"=>7, "ch"=>3, "d"=>5, "dcl"=>6, "dh"=>4, "dx"=>6, "eh"=>12, "el"=>7, "em"=>9, "en"=>8, "eng"=>10, "epi"=>8, "er"=>13, "ey"=>12, "f"=>2, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>6, "hv"=>7, "ih"=>11, "ix"=>12, "iy"=>10, "jh"=>4, "k"=>5, "kcl"=>6, "l"=>6, "m"=>8, "n"=>7, "ng"=>9, "nx"=>8, "ow"=>12, "oy"=>14, "p"=>4, "pau"=>8, "pcl"=>5, "q"=>8, "r"=>9, "s"=>0, "sh"=>2, "t"=>3, "tcl"=>4, "th"=>2, "uh"=>12, "uw"=>11, "ux"=>12, "v"=>4, "w"=>9, "y"=>7, "z"=>2, "zh"=>4, "DEL"=>5 },
  "sh" => { "aa"=>13, "ae"=>13, "ah"=>10, "ao"=>12, "aw"=>15, "ax"=>11, "ax-h"=>10, "axr"=>13, "ay"=>15, "b"=>8, "bcl"=>9, "ch"=>1, "d"=>7, "dcl"=>8, "dh"=>6, "dx"=>8, "eh"=>12, "el"=>9, "em"=>11, "en"=>10, "eng"=>10, "epi"=>8, "er"=>13, "ey"=>12, "f"=>4, "g"=>7, "gcl"=>8, "h#"=>8, "hh"=>6, "hv"=>7, "ih"=>11, "ix"=>12, "iy"=>10, "jh"=>2, "k"=>5, "kcl"=>6, "l"=>8, "m"=>10, "n"=>9, "ng"=>9, "nx"=>10, "ow"=>12, "oy"=>14, "p"=>6, "pau"=>8, "pcl"=>7, "q"=>8, "r"=>9, "s"=>2, "sh"=>0, "t"=>5, "tcl"=>6, "th"=>4, "uh"=>12, "uw"=>11, "ux"=>12, "v"=>6, "w"=>7, "y"=>5, "z"=>4, "zh"=>2, "DEL"=>5 },
  "t" => { "aa"=>10, "ae"=>10, "ah"=>7, "ao"=>9, "aw"=>12, "ax"=>8, "ax-h"=>7, "axr"=>10, "ay"=>12, "b"=>3, "bcl"=>4, "ch"=>4, "d"=>2, "dcl"=>3, "dh"=>3, "dx"=>3, "eh"=>9, "el"=>6, "em"=>6, "en"=>5, "eng"=>9, "epi"=>5, "er"=>10, "ey"=>9, "f"=>3, "g"=>6, "gcl"=>7, "h#"=>5, "hh"=>5, "hv"=>6, "ih"=>10, "ix"=>9, "iy"=>9, "jh"=>5, "k"=>4, "kcl"=>5, "l"=>5, "m"=>5, "n"=>4, "ng"=>8, "nx"=>5, "ow"=>9, "oy"=>11, "p"=>1, "pau"=>5, "pcl"=>2, "q"=>5, "r"=>8, "s"=>3, "sh"=>5, "t"=>0, "tcl"=>1, "th"=>1, "uh"=>11, "uw"=>10, "ux"=>11, "v"=>5, "w"=>10, "y"=>8, "z"=>5, "zh"=>7, "DEL"=>5 },
  "tcl" => { "aa"=>11, "ae"=>11, "ah"=>8, "ao"=>10, "aw"=>13, "ax"=>9, "ax-h"=>8, "axr"=>11, "ay"=>13, "b"=>4, "bcl"=>3, "ch"=>5, "d"=>3, "dcl"=>2, "dh"=>4, "dx"=>4, "eh"=>10, "el"=>7, "em"=>7, "en"=>6, "eng"=>10, "epi"=>4, "er"=>11, "ey"=>10, "f"=>4, "g"=>7, "gcl"=>6, "h#"=>4, "hh"=>6, "hv"=>7, "ih"=>11, "ix"=>10, "iy"=>10, "jh"=>6, "k"=>5, "kcl"=>4, "l"=>6, "m"=>6, "n"=>5, "ng"=>9, "nx"=>6, "ow"=>10, "oy"=>12, "p"=>2, "pau"=>4, "pcl"=>1, "q"=>4, "r"=>9, "s"=>4, "sh"=>6, "t"=>1, "tcl"=>0, "th"=>2, "uh"=>12, "uw"=>11, "ux"=>12, "v"=>6, "w"=>11, "y"=>9, "z"=>6, "zh"=>8, "DEL"=>5 },
  "th" => { "aa"=>11, "ae"=>11, "ah"=>8, "ao"=>10, "aw"=>13, "ax"=>9, "ax-h"=>8, "axr"=>11, "ay"=>13, "b"=>4, "bcl"=>5, "ch"=>5, "d"=>3, "dcl"=>4, "dh"=>2, "dx"=>4, "eh"=>10, "el"=>5, "em"=>7, "en"=>6, "eng"=>10, "epi"=>6, "er"=>11, "ey"=>10, "f"=>2, "g"=>7, "gcl"=>8, "h#"=>6, "hh"=>4, "hv"=>5, "ih"=>11, "ix"=>10, "iy"=>10, "jh"=>6, "k"=>5, "kcl"=>6, "l"=>4, "m"=>6, "n"=>5, "ng"=>9, "nx"=>6, "ow"=>10, "oy"=>12, "p"=>2, "pau"=>6, "pcl"=>3, "q"=>6, "r"=>7, "s"=>2, "sh"=>4, "t"=>1, "tcl"=>2, "th"=>0, "uh"=>12, "uw"=>11, "ux"=>12, "v"=>4, "w"=>9, "y"=>7, "z"=>4, "zh"=>6, "DEL"=>5 },
  "uh" => { "aa"=>3, "ae"=>5, "ah"=>4, "ao"=>2, "aw"=>7, "ax"=>3, "ax-h"=>4, "axr"=>3, "ay"=>7, "b"=>8, "bcl"=>9, "ch"=>11, "d"=>9, "dcl"=>10, "dh"=>10, "dx"=>10, "eh"=>4, "el"=>9, "em"=>7, "en"=>8, "eng"=>4, "epi"=>8, "er"=>5, "ey"=>6, "f"=>12, "g"=>5, "gcl"=>6, "h#"=>8, "hh"=>10, "hv"=>9, "ih"=>3, "ix"=>4, "iy"=>4, "jh"=>10, "k"=>7, "kcl"=>8, "l"=>10, "m"=>8, "n"=>9, "ng"=>5, "nx"=>10, "ow"=>2, "oy"=>2, "p"=>10, "pau"=>8, "pcl"=>11, "q"=>8, "r"=>7, "s"=>12, "sh"=>12, "t"=>11, "tcl"=>12, "th"=>12, "uh"=>0, "uw"=>1, "ux"=>2, "v"=>10, "w"=>5, "y"=>7, "z"=>10, "zh"=>10, "DEL"=>5 },
  "uw" => { "aa"=>4, "ae"=>6, "ah"=>3, "ao"=>1, "aw"=>8, "ax"=>4, "ax-h"=>5, "axr"=>4, "ay"=>8, "b"=>9, "bcl"=>10, "ch"=>10, "d"=>10, "dcl"=>11, "dh"=>11, "dx"=>11, "eh"=>5, "el"=>8, "em"=>8, "en"=>9, "eng"=>5, "epi"=>9, "er"=>4, "ey"=>5, "f"=>11, "g"=>6, "gcl"=>7, "h#"=>9, "hh"=>11, "hv"=>10, "ih"=>4, "ix"=>5, "iy"=>3, "jh"=>9, "k"=>6, "kcl"=>7, "l"=>9, "m"=>9, "n"=>10, "ng"=>6, "nx"=>11, "ow"=>1, "oy"=>3, "p"=>9, "pau"=>9, "pcl"=>10, "q"=>9, "r"=>8, "s"=>11, "sh"=>11, "t"=>10, "tcl"=>11, "th"=>11, "uh"=>1, "uw"=>0, "ux"=>1, "v"=>11, "w"=>6, "y"=>8, "z"=>11, "zh"=>11, "DEL"=>5 },
  "ux" => { "aa"=>5, "ae"=>7, "ah"=>4, "ao"=>2, "aw"=>7, "ax"=>5, "ax-h"=>6, "axr"=>5, "ay"=>7, "b"=>10, "bcl"=>11, "ch"=>11, "d"=>11, "dcl"=>12, "dh"=>12, "dx"=>12, "eh"=>6, "el"=>9, "em"=>9, "en"=>10, "eng"=>6, "epi"=>10, "er"=>3, "ey"=>6, "f"=>12, "g"=>7, "gcl"=>8, "h#"=>10, "hh"=>12, "hv"=>11, "ih"=>5, "ix"=>6, "iy"=>4, "jh"=>10, "k"=>7, "kcl"=>8, "l"=>10, "m"=>10, "n"=>11, "ng"=>7, "nx"=>12, "ow"=>2, "oy"=>4, "p"=>10, "pau"=>10, "pcl"=>11, "q"=>10, "r"=>9, "s"=>12, "sh"=>12, "t"=>11, "tcl"=>12, "th"=>12, "uh"=>2, "uw"=>1, "ux"=>0, "v"=>12, "w"=>7, "y"=>9, "z"=>12, "zh"=>12, "DEL"=>5 },
  "v" => { "aa"=>9, "ae"=>9, "ah"=>8, "ao"=>10, "aw"=>11, "ax"=>7, "ax-h"=>8, "axr"=>9, "ay"=>11, "b"=>2, "bcl"=>3, "ch"=>7, "d"=>3, "dcl"=>4, "dh"=>2, "dx"=>4, "eh"=>8, "el"=>7, "em"=>5, "en"=>6, "eng"=>8, "epi"=>6, "er"=>11, "ey"=>10, "f"=>2, "g"=>5, "gcl"=>6, "h#"=>6, "hh"=>4, "hv"=>3, "ih"=>9, "ix"=>8, "iy"=>10, "jh"=>6, "k"=>7, "kcl"=>8, "l"=>6, "m"=>4, "n"=>5, "ng"=>7, "nx"=>6, "ow"=>10, "oy"=>10, "p"=>4, "pau"=>6, "pcl"=>5, "q"=>6, "r"=>7, "s"=>4, "sh"=>6, "t"=>5, "tcl"=>6, "th"=>4, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>0, "w"=>7, "y"=>5, "z"=>2, "zh"=>4, "DEL"=>5 },
  "w" => { "aa"=>8, "ae"=>10, "ah"=>9, "ao"=>7, "aw"=>12, "ax"=>8, "ax-h"=>9, "axr"=>8, "ay"=>12, "b"=>7, "bcl"=>8, "ch"=>8, "d"=>8, "dcl"=>9, "dh"=>7, "dx"=>9, "eh"=>9, "el"=>10, "em"=>8, "en"=>9, "eng"=>5, "epi"=>9, "er"=>10, "ey"=>11, "f"=>9, "g"=>4, "gcl"=>5, "h#"=>9, "hh"=>7, "hv"=>6, "ih"=>8, "ix"=>9, "iy"=>9, "jh"=>7, "k"=>6, "kcl"=>7, "l"=>9, "m"=>7, "n"=>8, "ng"=>4, "nx"=>9, "ow"=>7, "oy"=>7, "p"=>9, "pau"=>9, "pcl"=>10, "q"=>7, "r"=>6, "s"=>9, "sh"=>7, "t"=>10, "tcl"=>11, "th"=>9, "uh"=>5, "uw"=>6, "ux"=>7, "v"=>7, "w"=>0, "y"=>2, "z"=>7, "zh"=>5, "DEL"=>5 },
  "y" => { "aa"=>8, "ae"=>8, "ah"=>7, "ao"=>9, "aw"=>10, "ax"=>6, "ax-h"=>7, "axr"=>8, "ay"=>10, "b"=>5, "bcl"=>6, "ch"=>6, "d"=>6, "dcl"=>7, "dh"=>5, "dx"=>7, "eh"=>7, "el"=>8, "em"=>6, "en"=>7, "eng"=>5, "epi"=>7, "er"=>10, "ey"=>9, "f"=>7, "g"=>4, "gcl"=>5, "h#"=>7, "hh"=>5, "hv"=>4, "ih"=>6, "ix"=>7, "iy"=>7, "jh"=>5, "k"=>6, "kcl"=>7, "l"=>7, "m"=>5, "n"=>6, "ng"=>4, "nx"=>7, "ow"=>9, "oy"=>9, "p"=>7, "pau"=>7, "pcl"=>8, "q"=>7, "r"=>6, "s"=>7, "sh"=>5, "t"=>8, "tcl"=>9, "th"=>7, "uh"=>7, "uw"=>8, "ux"=>9, "v"=>5, "w"=>2, "y"=>0, "z"=>5, "zh"=>3, "DEL"=>5 },
  "z" => { "aa"=>11, "ae"=>11, "ah"=>10, "ao"=>12, "aw"=>13, "ax"=>9, "ax-h"=>10, "axr"=>11, "ay"=>13, "b"=>4, "bcl"=>5, "ch"=>5, "d"=>3, "dcl"=>4, "dh"=>2, "dx"=>4, "eh"=>10, "el"=>7, "em"=>7, "en"=>6, "eng"=>8, "epi"=>8, "er"=>13, "ey"=>12, "f"=>4, "g"=>5, "gcl"=>6, "h#"=>8, "hh"=>6, "hv"=>5, "ih"=>9, "ix"=>10, "iy"=>10, "jh"=>4, "k"=>7, "kcl"=>8, "l"=>6, "m"=>6, "n"=>5, "ng"=>7, "nx"=>6, "ow"=>12, "oy"=>12, "p"=>6, "pau"=>8, "pcl"=>7, "q"=>8, "r"=>7, "s"=>2, "sh"=>4, "t"=>5, "tcl"=>6, "th"=>4, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>2, "w"=>7, "y"=>5, "z"=>0, "zh"=>2, "DEL"=>5 },
  "zh" => { "aa"=>11, "ae"=>11, "ah"=>10, "ao"=>12, "aw"=>13, "ax"=>9, "ax-h"=>10, "axr"=>11, "ay"=>13, "b"=>6, "bcl"=>7, "ch"=>3, "d"=>5, "dcl"=>6, "dh"=>4, "dx"=>6, "eh"=>10, "el"=>9, "em"=>9, "en"=>8, "eng"=>8, "epi"=>8, "er"=>13, "ey"=>12, "f"=>6, "g"=>5, "gcl"=>6, "h#"=>8, "hh"=>6, "hv"=>5, "ih"=>9, "ix"=>10, "iy"=>10, "jh"=>2, "k"=>7, "kcl"=>8, "l"=>8, "m"=>8, "n"=>7, "ng"=>7, "nx"=>8, "ow"=>12, "oy"=>12, "p"=>8, "pau"=>8, "pcl"=>9, "q"=>8, "r"=>7, "s"=>4, "sh"=>2, "t"=>7, "tcl"=>8, "th"=>6, "uh"=>10, "uw"=>11, "ux"=>12, "v"=>4, "w"=>5, "y"=>3, "z"=>2, "zh"=>0, "DEL"=>5 },
  "INS" => { "aa"=>5, "ae"=>5, "ah"=>5, "ao"=>5, "aw"=>5, "ax"=>5, "ax-h"=>5, "axr"=>5, "ay"=>5, "b"=>5, "bcl"=>5, "ch"=>5, "d"=>5, "dcl"=>5, "dh"=>5, "dx"=>5, "eh"=>5, "el"=>5, "em"=>5, "en"=>5, "eng"=>5, "epi"=>5, "er"=>5, "ey"=>5, "f"=>5, "g"=>5, "gcl"=>5, "h#"=>5, "hh"=>5, "hv"=>5, "ih"=>5, "ix"=>5, "iy"=>5, "jh"=>5, "k"=>5, "kcl"=>5, "l"=>5, "m"=>5, "n"=>5, "ng"=>5, "nx"=>5, "ow"=>5, "oy"=>5, "p"=>5, "pau"=>5, "pcl"=>5, "q"=>5, "r"=>5, "s"=>5, "sh"=>5, "t"=>5, "tcl"=>5, "th"=>5, "uh"=>5, "uw"=>5, "ux"=>5, "v"=>5, "w"=>5, "y"=>5, "z"=>5, "zh"=>5, "DEL"=>24 }
);

1; #make require happy

